@@ -1,3 +1,27 @@
+/*
+ * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ *
+ * @APPLE_LICENSE_HEADER_START@
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
+ * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
+ * @APPLE_LICENSE_HEADER_END@
+ */
 
 /*
  * mslplib.c : Minimal SLP v2 User Agent implementation.
@@ -26,6 +50,9 @@
  * (c) Sun Microsystems, 1998, All Rights Reserved.
  * Author: Erik Guttman
  */
+ /*
+	Portions Copyright (c) 2002 Apple Computer, Inc. All rights reserved.
+ */
 
 #include <stdio.h>    /* these definitions are standard C, system indep. */
 #include <time.h>
@@ -69,9 +96,15 @@ EXPORT const char * get_fun_str(int i) {
 
 EXPORT int GetSLPPort(void)
 {
-    char*	endPtr = NULL;
-	int	port = strtol(SLPGetProperty("com.apple.slp.port"),&endPtr,10);
-    
+	int		port = 427;		// default port
+	const char*	slpProperty = SLPGetProperty("com.apple.slp.port");
+	
+	if ( slpProperty )
+	{
+		char*	endPtr = NULL;
+		strtol(slpProperty,&endPtr,10);
+    }
+	
     return port;
 }
 
@@ -204,7 +237,7 @@ TESTEXPORT SLPInternalError process_reply(const char *pcSendBuf,
                 SLPScopeCallback *pssc = (SLPScopeCallback*) pvCallback;
         
                 if ( pcSendBuf && pcRecvBuf )	// its ok to have a nil pcSendBuf and pcRecvBuf, it just means the search
-                {				// is finished so we don't want to log an error!  // KA 4/19/00
+                {				// is finished so we don't want to log an error!
                     err = get_header(pcSendBuf,pcRecvBuf,iRecvSz,&slph,&offset);
                     
                     if ( err != SLP_OK )
@@ -238,7 +271,7 @@ TESTEXPORT SLPInternalError process_reply(const char *pcSendBuf,
                 if ( err == SLP_OK /*&& slph.h_ucFun == SAADVERT*/ )
                     (void) pssc (hSLP, pcScope, SLP_LAST_CALL, pvUser);
         
-                return err;			// we should really be returning the error here // KA 4/19/00
+                return err;			// we should really be returning the error here
             }
         #endif	/* MAC_OS_X */
             default:
